#include <windows.h> 
#include "gpio.h"

HANDLE  hGpio = NULL;       ///< handle to the GPIO library
///< Demo menu option enum
typedef enum{
	OPTION_INPUT =1,
	OPTION_OUTPUT,
	OPTION_EXIT,
	OPTION_INVALID
}OPTION_FOR_SELECTION;

//******************************************************************************
/// Run gpio demo as output configuration, toggle sodimm number 50 times with 500ms delay
/// @param[in]          sodimm         Sodimm Pin Number               
/// @retval             TRUE           Success
/// @retval             FALSE          Error
BOOL functionOutputDemo(DWORD sodimm)
{
	uIo     io = COLIBRI_PIN(sodimm);
	DWORD loopCount		= 0;				///< Counting variable
	printf("GPIO as Output\n");
	Gpio_ConfigureAsGpio(hGpio, io);		///< Set SODIMM Pin  as GPIO 
	Gpio_SetDir(hGpio, io, ioOutput);		///< Set SODIMM Pin direction as Output
	for (loopCount = 0; loopCount < 50; loopCount++)
	{
		printf("Setting Sodimm %d low\n", sodimm);
		Gpio_SetLevel(hGpio, io, ioLow);     ///< Set the logic level of SODIMM pin  to low
		Sleep(500);						///< Delay by 500 ms  
		printf("Setting Sodimm %d high\n", sodimm);
		Gpio_SetLevel(hGpio, io, ioHigh);      ///< Set the logic level of SODIMM pin  to high
		Sleep(500);						///< Delay by 500 ms
	}
	return TRUE;
}
//******************************************************************************
/// Run gpio demo as Input configuration and return sodimm level 0 for low, 1 for high
/// @param[in]          sodimm         Sodimm Pin Number               
/// @retval             TRUE           Success
/// @retval             FALSE          Error
void functionInputDemo(DWORD sodimm)
{
	uIo			io			= COLIBRI_PIN(sodimm);
	DWORD		loopCount	= 0;					///< Counting variable
	tIoLevel	inputVal	= ioLow;				///< Variable to store input status
	printf("GPIO as Input\n");
	Gpio_ConfigureAsGpio(hGpio, io);				///< Set SODIMM Pin  as GPIO  
	Gpio_SetDir(hGpio, io, ioInput);				///< Set SODIMM Pin direction as Output
	while(loopCount < 10)
	{
		inputVal = Gpio_GetLevel(hGpio, io);		///< Get the logic level of SODIMM pin 
		if(inputVal == ioHigh)						///< Print message every time sodimm pin toggle high
		{
			printf("SODIMM pin 101 level: %d\n", inputVal);
			loopCount++;
			Sleep(100);
		}
	}
}
//******************************************************************************
/// Get Sodimm number from console                
/// @retval             sodimm			Success
/// @retval             OPTION_INVALID  Error
DWORD getSodimmNumber()
{
	DWORD sodimmNumber	= 0;
	printf("Enter Sodimm Number\r\n");
	fflush(stdin);
	scanf("%d", &sodimmNumber);						///< Get Sodimm number from console window
	if(sodimmNumber <=0 || sodimmNumber > 300)		///< Check for invalid sodimm number/MXM number 
	{
		printf("Invalid Sodimm number\r\n");
		return OPTION_INVALID;
	}
	return sodimmNumber;
}
int wmain()
{ 
	DWORD selection		= OPTION_INVALID;
	BOOL    success;
	hGpio = Gpio_Init(NULL);						///< Init GPIO library
	ASSERT(hGpio != 0);
	success = Gpio_Open(hGpio);
	ASSERT (success);
	printf("Demo application to show GPIO as Output and Input\r\n");
	while (selection != OPTION_EXIT)
	{
		printf("\n\n********************** Option **********************");
		printf("\n1. Input \n2. Output \n3. Exit\t ");
		fflush(stdin);
		scanf_s("%u", &selection);
		switch(selection)
		{
		case OPTION_INPUT:							///< option to run demo as Input configuration
			functionInputDemo(getSodimmNumber());	
			break;
		case OPTION_OUTPUT:							///< option to run demo as Output configuration
			functionOutputDemo(getSodimmNumber());
			break;
		case OPTION_EXIT:
			success = Gpio_Close(hGpio);			///< Release resources used by GPIO lib and de-initialize GPIO library function
			ASSERT(success);
			success = Gpio_Deinit(hGpio);
			ASSERT(success);
			break;
		default:
			printf("\nInvalid selection. Try again.");
		}
	}
	return(TRUE);
}